@@ -20,8 +20,8 @@ from __future__ import annotations
 import datetime
 
 from airflow.providers.standard.sensors.date_time import DateTimeSensorAsync
+from airflow.sdk import timezone
 from airflow.sdk.definitions.dag import dag
-from airflow.utils import timezone
 
 
 @dag
